/*
 Copyright (c) 2008-2009 TrueCrypt Developers Association. All rights reserved.

 Governed by the TrueCrypt License 3.0 the full text of which is contained in
 the file License.txt included in TrueCrypt binary and source code distribution
 packages.
*/

#ifndef GST_HEADER_Boot_BootDefs
#define GST_HEADER_Boot_BootDefs

// Total memory required (CODE + DATA + BSS + STACK + 0x100) in KBytes - determined from linker map.
#define GST__BOOT_MEMORY_REQUIRED	40

#ifdef GST_WINDOWS_BOOT_SINGLE_CIPHER_MODE

/*
#	undef GST__BOOT_MEMORY_REQUIRED

#	ifdef GST_WINDOWS_BOOT_GOST
#		ifdef GST_WINDOWS_BOOT_RESCUE_DISK_MODE
#			define GST__BOOT_MEMORY_REQUIRED xx
#		else
#			define GST__BOOT_MEMORY_REQUIRED ..
#		endif
#	endif
*/

#if 0
#	undef GST__BOOT_MEMORY_REQUIRED
#	define GST__BOOT_MEMORY_REQUIRED 60
#endif

#endif

// Modifying this value can introduce incompatibility with previous versions
#define GST__BOOT_LOADER_SEGMENT			GST_HEX (9000)	// Some buggy BIOS routines fail if CS bits 0-10 are not zero

#if GST__BOOT_MEMORY_REQUIRED <= 32
#	define GST__BOOT_LOADER_SEGMENT_LOW	(GST__BOOT_LOADER_SEGMENT - 32 * 1024 / 16)	
#else
#	define GST__BOOT_LOADER_SEGMENT_LOW	(GST__BOOT_LOADER_SEGMENT - 64 * 1024 / 16)	
#endif

#define GST__COM_EXECUTABLE_OFFSET		GST_HEX (100)

#define GST__BOOT_LOADER_LOWMEM_SEGMENT	GST_HEX (2000)
#define GST__BOOT_LOADER_BUFFER_SEGMENT	GST_HEX (4000)
#define GST__BOOT_LOADER_ALT_SEGMENT		GST_HEX (6000)

#define GST__BOOT_LOADER_STACK_TOP (GST_BOOT_MEMORY_REQUIRED * GST_UNSIGNED (1024) - 4)

#define GST__LB_SIZE 512
#define GST__BOOT_LOADER_AREA_SECTOR_COUNT 63

#define GST__BOOT_SECTOR_VERSION_OFFSET 430
#define GST__BOOT_SECTOR_LOADER_LENGTH_OFFSET 432
#define GST__BOOT_SECTOR_LOADER_CHECKSUM_OFFSET 434
#define GST__BOOT_SECTOR_USER_CONFIG_OFFSET 438
#define GST__BOOT_SECTOR_CONFIG_OFFSET 439	// The last byte that is reserved for the boot loader

#define GST__BOOT_SECTOR_USER_MESSAGE_MAX_LENGTH 24
#define GST__BOOT_SECTOR_USER_MESSAGE_OFFSET (GST__BOOT_SECTOR_VERSION_OFFSET - GST__BOOT_SECTOR_USER_MESSAGE_MAX_LENGTH)

#define GST__BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_SIZE 4
#define GST__BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_OFFSET (GST__BOOT_SECTOR_USER_MESSAGE_OFFSET - GST__BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_SIZE)

#define GST__BOOT_LOADER_DECOMPRESSOR_START_SECTOR 2
#define GST__BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT 4
#define GST__BOOT_LOADER_DECOMPRESSOR_MEMORY_SIZE 32768
#define GST__BOOT_LOADER_COMPRESSED_BUFFER_OFFSET (GST_COM_EXECUTABLE_OFFSET + 3072)

#define GST__BOOT_LOADER_START_SECTOR (GST_BOOT_LOADER_DECOMPRESSOR_START_SECTOR + GST_BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT)
#define GST__MAX_BOOT_LOADER_SECTOR_COUNT (GST_BOOT_LOADER_AREA_SECTOR_COUNT - GST_BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT - 2)
#define GST__MAX_BOOT_LOADER_DECOMPRESSED_SIZE ((GST_BOOT_LOADER_AREA_SECTOR_COUNT - 2) * GST_LB_SIZE)

#define GST__BOOT_LOADER_BACKUP_SECTOR_COUNT 30

#define GST__GZIP_HEADER_SIZE 10

#define GST__BOOT_CFG_FLAG_AREA_SIZE	1	// In bytes

// If you add more flags, revise GST__BOOT_CFG_FLAG_AREA_SIZE
#define GST__BOOT_CFG_FLAG_BACKUP_LOADER_AVAILABLE		GST_HEX (02)
#define GST__BOOT_CFG_FLAG_WINDOWS_VISTA_OR_LATER		GST_HEX (04)
#define GST__BOOT_CFG_FLAG_RESCUE_DISABLE_HW_ENCRYPTION	GST_HEX (10)
#define GST__BOOT_CFG_FLAG_RESCUE_DISK_ORIG_SYS_LOADER	GST_HEX (20)
#define GST__BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE		(GST_HEX (40) + GST_HEX (80))

// Modifying the following values can introduce incompatibility with previous versions
#define GST__BOOT_USER_CFG_FLAG_SILENT_MODE				GST_HEX (01)
#define GST__BOOT_USER_CFG_FLAG_DISABLE_ESC				GST_HEX (02)
#define GST__BOOT_USER_CFG_FLAG_DISABLE_HW_ENCRYPTION	GST_HEX (04)

// The following items are treated as a 2-bit value (apply GST_BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE to obtain the value)
#define GST__HIDDEN_OS_CREATION_PHASE_NONE		0
#define GST__HIDDEN_OS_CREATION_PHASE_CLONING	GST_HEX (40)		// The boot loader is to copy the content of the system partition to the hidden volume
#define GST__HIDDEN_OS_CREATION_PHASE_WIPING		GST_HEX (80)		// The boot loader has successfully copied the content of the system partition to the hidden volume. The original OS is to be wiped now.
#define GST__HIDDEN_OS_CREATION_PHASE_WIPED		(GST_HEX (40) + GST_HEX (80))	// The original OS has been wiped. The user is required to install a new OS (decoy OS) on the system partition now.


#ifdef GST_ASM_PREPROCESS

#define GST_HEX(N) 0##N##h
#define GST_UNSIGNED(N) N

GST_BOOT_MEMORY_REQUIRED = GST__BOOT_MEMORY_REQUIRED
GST_BOOT_LOADER_SEGMENT = GST__BOOT_LOADER_SEGMENT
GST_BOOT_LOADER_SEGMENT_LOW = GST__BOOT_LOADER_SEGMENT_LOW
GST_COM_EXECUTABLE_OFFSET = GST__COM_EXECUTABLE_OFFSET
GST_BOOT_LOADER_LOWMEM_SEGMENT = GST__BOOT_LOADER_LOWMEM_SEGMENT
GST_BOOT_LOADER_BUFFER_SEGMENT = GST__BOOT_LOADER_BUFFER_SEGMENT
GST_BOOT_LOADER_ALT_SEGMENT = GST__BOOT_LOADER_ALT_SEGMENT
GST_BOOT_LOADER_STACK_TOP = GST__BOOT_LOADER_STACK_TOP
GST_LB_SIZE = GST__LB_SIZE
GST_BOOT_LOADER_AREA_SECTOR_COUNT = GST__BOOT_LOADER_AREA_SECTOR_COUNT
GST_BOOT_SECTOR_LOADER_LENGTH_OFFSET = GST__BOOT_SECTOR_LOADER_LENGTH_OFFSET
GST_BOOT_SECTOR_LOADER_CHECKSUM_OFFSET = GST__BOOT_SECTOR_LOADER_CHECKSUM_OFFSET
GST_BOOT_SECTOR_CONFIG_OFFSET = GST__BOOT_SECTOR_CONFIG_OFFSET
GST_BOOT_SECTOR_USER_CONFIG_OFFSET = GST__BOOT_SECTOR_USER_CONFIG_OFFSET
GST_BOOT_LOADER_DECOMPRESSOR_START_SECTOR = GST__BOOT_LOADER_DECOMPRESSOR_START_SECTOR
GST_BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT = GST__BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT
GST_BOOT_LOADER_DECOMPRESSOR_MEMORY_SIZE = GST__BOOT_LOADER_DECOMPRESSOR_MEMORY_SIZE
GST_BOOT_LOADER_COMPRESSED_BUFFER_OFFSET = GST__BOOT_LOADER_COMPRESSED_BUFFER_OFFSET
GST_BOOT_LOADER_START_SECTOR = GST__BOOT_LOADER_START_SECTOR
GST_MAX_BOOT_LOADER_SECTOR_COUNT = GST__MAX_BOOT_LOADER_SECTOR_COUNT
GST_MAX_BOOT_LOADER_DECOMPRESSED_SIZE = GST__MAX_BOOT_LOADER_DECOMPRESSED_SIZE
GST_BOOT_LOADER_BACKUP_SECTOR_COUNT = GST__BOOT_LOADER_BACKUP_SECTOR_COUNT
GST_GZIP_HEADER_SIZE = GST__GZIP_HEADER_SIZE
GST_BOOT_CFG_FLAG_AREA_SIZE = GST__BOOT_CFG_FLAG_AREA_SIZE
GST_BOOT_CFG_FLAG_BACKUP_LOADER_AVAILABLE = GST__BOOT_CFG_FLAG_BACKUP_LOADER_AVAILABLE
GST_BOOT_CFG_FLAG_WINDOWS_VISTA_OR_LATER = GST__BOOT_CFG_FLAG_WINDOWS_VISTA_OR_LATER
GST_BOOT_CFG_FLAG_RESCUE_DISK_ORIG_SYS_LOADER = GST__BOOT_CFG_FLAG_RESCUE_DISK_ORIG_SYS_LOADER
GST_BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE = GST__BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE
GST_BOOT_USER_CFG_FLAG_SILENT_MODE = GST__BOOT_USER_CFG_FLAG_SILENT_MODE
GST_HIDDEN_OS_CREATION_PHASE_NONE = GST__HIDDEN_OS_CREATION_PHASE_NONE
GST_HIDDEN_OS_CREATION_PHASE_CLONING = GST__HIDDEN_OS_CREATION_PHASE_CLONING
GST_HIDDEN_OS_CREATION_PHASE_WIPING = GST__HIDDEN_OS_CREATION_PHASE_WIPING
GST_HIDDEN_OS_CREATION_PHASE_WIPED = GST__HIDDEN_OS_CREATION_PHASE_WIPED

#else // GST_ASM_PREPROCESS

#define GST_HEX(N) 0x##N
#define GST_UNSIGNED(N) N##U

#define GST_BOOT_MEMORY_REQUIRED GST__BOOT_MEMORY_REQUIRED
#define GST_BOOT_LOADER_SEGMENT GST__BOOT_LOADER_SEGMENT
#define GST_COM_EXECUTABLE_OFFSET GST__COM_EXECUTABLE_OFFSET
#define GST_BOOT_LOADER_LOWMEM_SEGMENT GST__BOOT_LOADER_LOWMEM_SEGMENT
#define GST_BOOT_LOADER_BUFFER_SEGMENT GST__BOOT_LOADER_BUFFER_SEGMENT
#define GST_BOOT_LOADER_ALT_SEGMENT GST__BOOT_LOADER_ALT_SEGMENT
#define GST_BOOT_LOADER_STACK_TOP (GST__BOOT_LOADER_STACK_TOP)
#define GST_BOOT_LOADER_AREA_SECTOR_COUNT GST__BOOT_LOADER_AREA_SECTOR_COUNT
#define GST_BOOT_SECTOR_USER_MESSAGE_OFFSET GST__BOOT_SECTOR_USER_MESSAGE_OFFSET
#define GST_BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_SIZE GST__BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_SIZE
#define GST_BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_OFFSET GST__BOOT_SECTOR_OUTER_VOLUME_BAK_HEADER_CRC_OFFSET
#define GST_BOOT_SECTOR_USER_MESSAGE_MAX_LENGTH GST__BOOT_SECTOR_USER_MESSAGE_MAX_LENGTH
#define GST_BOOT_SECTOR_VERSION_OFFSET GST__BOOT_SECTOR_VERSION_OFFSET
#define GST_BOOT_SECTOR_LOADER_LENGTH_OFFSET GST__BOOT_SECTOR_LOADER_LENGTH_OFFSET
#define GST_BOOT_SECTOR_LOADER_CHECKSUM_OFFSET GST__BOOT_SECTOR_LOADER_CHECKSUM_OFFSET
#define GST_BOOT_LOADER_DECOMPRESSOR_START_SECTOR GST__BOOT_LOADER_DECOMPRESSOR_START_SECTOR
#define GST_BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT GST__BOOT_LOADER_DECOMPRESSOR_SECTOR_COUNT
#define GST_BOOT_SECTOR_CONFIG_OFFSET GST__BOOT_SECTOR_CONFIG_OFFSET
#define GST_BOOT_SECTOR_USER_CONFIG_OFFSET GST__BOOT_SECTOR_USER_CONFIG_OFFSET
#define GST_BOOT_LOADER_START_SECTOR GST__BOOT_LOADER_START_SECTOR
#define GST_LB_SIZE GST__LB_SIZE
#define GST_MAX_BOOT_LOADER_SECTOR_COUNT GST__MAX_BOOT_LOADER_SECTOR_COUNT
#define GST_MAX_BOOT_LOADER_DECOMPRESSED_SIZE GST__MAX_BOOT_LOADER_DECOMPRESSED_SIZE
#define GST_BOOT_LOADER_BACKUP_SECTOR_COUNT GST__BOOT_LOADER_BACKUP_SECTOR_COUNT
#define GST_GZIP_HEADER_SIZE GST__GZIP_HEADER_SIZE
#define GST_BOOT_CFG_FLAG_AREA_SIZE GST__BOOT_CFG_FLAG_AREA_SIZE
#define GST_BOOT_CFG_FLAG_BACKUP_LOADER_AVAILABLE GST__BOOT_CFG_FLAG_BACKUP_LOADER_AVAILABLE
#define GST_BOOT_CFG_FLAG_WINDOWS_VISTA_OR_LATER GST__BOOT_CFG_FLAG_WINDOWS_VISTA_OR_LATER
#define GST_BOOT_CFG_FLAG_RESCUE_DISK_ORIG_SYS_LOADER GST__BOOT_CFG_FLAG_RESCUE_DISK_ORIG_SYS_LOADER
#define GST_BOOT_CFG_FLAG_RESCUE_DISABLE_HW_ENCRYPTION GST__BOOT_CFG_FLAG_RESCUE_DISABLE_HW_ENCRYPTION
#define GST_BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE GST__BOOT_CFG_MASK_HIDDEN_OS_CREATION_PHASE
#define GST_BOOT_USER_CFG_FLAG_SILENT_MODE GST__BOOT_USER_CFG_FLAG_SILENT_MODE
#define GST_BOOT_USER_CFG_FLAG_DISABLE_ESC GST__BOOT_USER_CFG_FLAG_DISABLE_ESC
#define GST_BOOT_USER_CFG_FLAG_DISABLE_HW_ENCRYPTION GST__BOOT_USER_CFG_FLAG_DISABLE_HW_ENCRYPTION
#define GST_HIDDEN_OS_CREATION_PHASE_NONE GST__HIDDEN_OS_CREATION_PHASE_NONE
#define GST_HIDDEN_OS_CREATION_PHASE_CLONING GST__HIDDEN_OS_CREATION_PHASE_CLONING
#define GST_HIDDEN_OS_CREATION_PHASE_WIPING GST__HIDDEN_OS_CREATION_PHASE_WIPING
#define GST_HIDDEN_OS_CREATION_PHASE_WIPED GST__HIDDEN_OS_CREATION_PHASE_WIPED

#endif // GST_ASM_PREPROCESS

#endif // GST_HEADER_Boot_BootDefs
